@@ -257,14 +257,15 @@ wxString CControlSocket::ConvertDomainName(wxString const& domain)
 {
 #ifdef __WXMSW__
 	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, 0, 0);
-	if( !len ) {
+	if (!len) {
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 
 	wchar_t* output = new wchar_t[len];
 	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, output, len);
-	if( !res ) {
+	if (!res) {
+		delete [] output;
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
@@ -918,6 +919,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 	case SocketEventType::connection_next:
 		if (error)
 			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
+		SetAlive();
 		break;
 	case SocketEventType::connection:
 		if (error) {
@@ -1044,7 +1046,7 @@ int CRealControlSocket::ContinueConnect()
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
-		LogMessage(MessageType::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost());
+		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->FormatHost(), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
 		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
 		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
@@ -1228,8 +1230,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			if (engine_.GetDirectoryCache().LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
 				matched_case)
 			{
-				wxLongLong size = entry.size;
-				pData->remoteFileSize = size.GetValue();
+				pData->remoteFileSize = entry.size;
 				if (entry.has_date())
 					pData->fileTime = entry.time;
 
