@@ -12,10 +12,15 @@
 #include "window_state_manager.h"
 #include "wrapengine.h"
 #include "xmlfunctions.h"
+#include "fzputtygen_interface.h"
 #include "xrc_helper.h"
 
 #include <wx/dcclient.h>
 #include <wx/dnd.h>
+#include <wx/file.h>
+#include <wx/gbsizer.h>
+
+#include <algorithm>
 
 #ifdef __WXMSW__
 #include "commctrl.h"
@@ -35,6 +40,7 @@ EVT_TREE_SEL_CHANGING(XRCID("ID_SITETREE"), CSiteManagerDialog::OnSelChanging)
 EVT_TREE_SEL_CHANGED(XRCID("ID_SITETREE"), CSiteManagerDialog::OnSelChanged)
 EVT_CHOICE(XRCID("ID_LOGONTYPE"), CSiteManagerDialog::OnLogontypeSelChanged)
 EVT_BUTTON(XRCID("ID_BROWSE"), CSiteManagerDialog::OnRemoteDirBrowse)
+EVT_BUTTON(XRCID("ID_KEYFILE_BROWSE"), CSiteManagerDialog::OnKeyFileBrowse)
 EVT_TREE_ITEM_ACTIVATED(XRCID("ID_SITETREE"), CSiteManagerDialog::OnItemActivated)
 EVT_CHECKBOX(XRCID("ID_LIMITMULTIPLE"), CSiteManagerDialog::OnLimitMultipleConnectionsChanged)
 EVT_RADIOBUTTON(XRCID("ID_CHARSET_AUTO"), CSiteManagerDialog::OnCharsetChange)
@@ -394,6 +400,16 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 	SetMinSize(GetSizer()->GetMinSize() + size - clientSize);
 	SetClientSize(minSize);
 
+	// Set min height of general page sizer
+	auto generalSizer = static_cast<wxGridBagSizer*>(xrc_call(*this, "ID_PROTOCOL", &wxWindow::GetContainingSizer));
+	generalSizer->SetMinSize(generalSizer->GetMinSize());
+	generalSizer->SetEmptyCellSize(wxSize(-5, -5));
+
+	// Set min height of encryption row
+	wxSize descSize = XRCCTRL(*this, "ID_ENCRYPTION_DESC", wxWindow)->GetSize();
+	wxSize encSize = XRCCTRL(*this, "ID_ENCRYPTION", wxWindow)->GetSize();
+	xrc_call(*this, "ID_ENCRYPTION", &wxWindow::GetContainingSizer)->GetItem(1)->SetMinSize(0, std::max(descSize.GetHeight(), encSize.GetHeight()));
+
 	// Load bookmark notebook
 	m_pNotebook_Bookmark = new wxNotebook(m_pNotebook_Site->GetParent(), -1);
 	wxPanel* pPanel = new wxPanel;
@@ -729,8 +745,8 @@ bool CSiteManagerDialog::Load()
 	pTree->SetItemImage(treeId, 1, wxTreeItemIcon_SelectedExpanded);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto document = file.Load();
+	if (!document) {
 		wxString msg = file.GetError() + _T("\n") + _("The Site Manager cannot be used unless the file gets repaired.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
@@ -744,8 +760,8 @@ bool CSiteManagerDialog::Load()
 		}
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return true;
 
 	wxString lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
@@ -759,7 +775,7 @@ bool CSiteManagerDialog::Load()
 		lastSelection.clear();
 	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection, false);
 
-	bool res = CSiteManager::Load(pElement, handler);
+	bool res = CSiteManager::Load(element, handler);
 
 	pTree->SortChildren(treeId);
 	pTree->Expand(treeId);
@@ -771,7 +787,7 @@ bool CSiteManagerDialog::Load()
 	return res;
 }
 
-bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wxTreeItemId()*/)
+bool CSiteManagerDialog::Save(pugi::xml_node element /*=0*/, wxTreeItemId treeId /*=wxTreeItemId()*/)
 {
 	if (!m_pSiteManagerMutex)
 		return false;
@@ -780,33 +796,32 @@ bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId
 	if (!pTree)
 		return false;
 
-	if (!pElement || !treeId)
-	{
+	if (!element || !treeId) {
 		// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 		// to the same file or one is reading while the other one writes.
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 		CXmlFile xml(wxGetApp().GetSettingsFile(_T("sitemanager")));
 
-		TiXmlElement* pDocument = xml.Load();
-		if (!pDocument) {
+		auto document = xml.Load();
+		if (!document) {
 			wxString msg = xml.GetError() + _T("\n") + _("Any changes made in the Site Manager could not be saved.");
 			wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 			return false;
 		}
 
-		TiXmlElement *pServers = pDocument->FirstChildElement("Servers");
-		while (pServers) {
-			pDocument->RemoveChild(pServers);
-			pServers = pDocument->FirstChildElement("Servers");
+		auto servers = document.child("Servers");
+		while (servers) {
+			document.remove_child(servers);
+			servers = document.child("Servers");
 		}
-		pElement = pDocument->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
+		element = document.append_child("Servers");
 
-		if (!pElement)
+		if (!element)
 			return true;
 
-		bool res = Save(pElement, m_ownSites);
+		bool res = Save(element, m_ownSites);
 
 		if (!xml.Save(false)) {
 			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
@@ -821,9 +836,8 @@ bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId
 	wxTreeItemId child;
 	wxTreeItemIdValue cookie;
 	child = pTree->GetFirstChild(treeId, cookie);
-	while (child.IsOk())
-	{
-		SaveChild(pElement, child);
+	while (child.IsOk()) {
+		SaveChild(element, child);
 
 		child = pTree->GetNextChild(treeId, cookie);
 	}
@@ -831,45 +845,41 @@ bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId
 	return false;
 }
 
-bool CSiteManagerDialog::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
+bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return false;
 
-	wxString name = pTree->GetItemText(child);
-	wxScopedCharBuffer utf8 = name.utf8_str();
+	wxString const name = pTree->GetItemText(child);
 
 	CSiteManagerItemData* data = static_cast<CSiteManagerItemData* >(pTree->GetItemData(child));
 	if (!data) {
-		TiXmlNode* pNode = pElement->LinkEndChild(new TiXmlElement("Folder"));
+		auto node = element.append_child("Folder");
 		const bool expanded = pTree->IsExpanded(child);
-		SetTextAttribute(pNode->ToElement(), "expanded", expanded ? _T("1") : _T("0"));
-
-		pNode->LinkEndChild(new TiXmlText(utf8));
-
-		Save(pNode->ToElement(), child);
+		SetTextAttribute(node, "expanded", expanded ? _T("1") : _T("0"));
+		AddTextElement(node, name);
+		Save(node, child);
 	}
 	else if (data->m_type == CSiteManagerItemData::SITE) {
 		CSiteManagerItemData_Site *site_data = static_cast<CSiteManagerItemData_Site* >(data);
-		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
-		SetServer(pNode, site_data->m_server);
+		auto node = element.append_child("Server");
+		SetServer(node, site_data->m_server);
 
 		// Save comments
-		AddTextElement(pNode, "Comments", site_data->m_comments);
+		AddTextElement(node, "Comments", site_data->m_comments);
 
 		// Save local dir
-		AddTextElement(pNode, "LocalDir", data->m_localDir);
+		AddTextElement(node, "LocalDir", data->m_localDir);
 
 		// Save remote dir
-		AddTextElement(pNode, "RemoteDir", data->m_remoteDir.GetSafePath());
+		AddTextElement(node, "RemoteDir", data->m_remoteDir.GetSafePath());
 
-		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
-		AddTextElementRaw(pNode, "DirectoryComparison", data->m_comparison ? "1" : "0");
+		AddTextElementRaw(node, "SyncBrowsing", data->m_sync ? "1" : "0");
+		AddTextElementRaw(node, "DirectoryComparison", data->m_comparison ? "1" : "0");
+		AddTextElement(node, name);
 
-		pNode->LinkEndChild(new TiXmlText(utf8));
-
-		Save(pNode, child);
+		Save(node, child);
 
 		if (site_data->connected_item != -1) {
 			if ((*m_connected_sites)[site_data->connected_item].server == site_data->m_server) {
@@ -879,18 +889,18 @@ bool CSiteManagerDialog::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 		}
 	}
 	else {
-		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
+		auto node = element.append_child("Bookmark");
 
-		AddTextElement(pNode, "Name", name);
+		AddTextElement(node, "Name", name);
 
 		// Save local dir
-		AddTextElement(pNode, "LocalDir", data->m_localDir);
+		AddTextElement(node, "LocalDir", data->m_localDir);
 
 		// Save remote dir
-		AddTextElement(pNode, "RemoteDir", data->m_remoteDir.GetSafePath());
+		AddTextElement(node, "RemoteDir", data->m_remoteDir.GetSafePath());
 
-		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
-		AddTextElementRaw(pNode, "DirectoryComparison", data->m_comparison ? "1" : "0");
+		AddTextElementRaw(node, "SyncBrowsing", data->m_sync ? "1" : "0");
+		AddTextElementRaw(node, "DirectoryComparison", data->m_comparison ? "1" : "0");
 	}
 
 	return true;
@@ -945,7 +955,7 @@ bool CSiteManagerDialog::Verify()
 			return false;
 		}
 
-		LogonType logon_type = CServer::GetLogonTypeFromName(XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection());
+		auto logon_type = GetLogonType();
 
 		ServerProtocol protocol = GetProtocol();
 		wxASSERT(protocol != UNKNOWN);
@@ -1020,7 +1030,7 @@ bool CSiteManagerDialog::Verify()
 			return false;
 		}
 
-		// The way TinyXML handles blanks, we can't use username of only spaces
+		// We don't allow username of only spaces, confuses both users and XML libraries
 		if (!user.empty()) {
 			bool space_only = true;
 			for (unsigned int i = 0; i < user.Len(); ++i) {
@@ -1045,6 +1055,26 @@ bool CSiteManagerDialog::Verify()
 			return false;
 		}
 
+		// In key file logon type, check that the provided key file exists
+		wxString keyFile, keyFileComment, keyFileData;
+		if (logon_type == KEY) {
+			keyFile = XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->GetValue();
+			if (keyFile.empty()) {
+				wxMessageBox(_("You have to enter a key file path"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				xrc_call(*this, "ID_KEYFILE", &wxWindow::SetFocus);
+				return false;
+			}
+
+			// Check (again) that the key file is in the correct format since it might have been introduced manually
+			CFZPuttyGenInterface cfzg(this);
+			if (cfzg.LoadKeyFile(keyFile, false, keyFileComment, keyFileData))
+				XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->ChangeValue(keyFile);
+			else {
+				xrc_call(*this, "ID_KEYFILE", &wxWindow::SetFocus);
+				return false;
+			}
+		}
+
 		const wxString remotePathRaw = XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->GetValue();
 		if (!remotePathRaw.empty()) {
 			const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
@@ -1273,9 +1303,13 @@ void CSiteManagerDialog::OnLogontypeSelChanged(wxCommandEvent& event)
 	if (!data)
 		return;
 
+	SetControlVisibility(GetProtocol(), GetLogonType());
+
 	XRCCTRL(*this, "ID_USER", wxTextCtrl)->Enable(event.GetString() != _("Anonymous"));
 	XRCCTRL(*this, "ID_PASS", wxTextCtrl)->Enable(event.GetString() == _("Normal") || event.GetString() == _("Account"));
 	XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->Enable(event.GetString() == _("Account"));
+	XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->Enable(event.GetString() == _("Key file"));
+	XRCCTRL(*this, "ID_KEYFILE_BROWSE", wxButton)->Enable(event.GetString() == _("Key file"));
 }
 
 bool CSiteManagerDialog::UpdateItem()
@@ -1337,13 +1371,15 @@ bool CSiteManagerDialog::UpdateServer(CSiteManagerItemData_Site &server, const w
 	}
 	server.m_server.SetHost(host, port);
 
-	enum LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
+	auto logon_type = GetLogonType();
 	server.m_server.SetLogonType(logon_type);
 
 	server.m_server.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue),
 							xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue));
 	server.m_server.SetAccount(xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue));
 
+	server.m_server.SetKeyFile(XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->GetValue());
+
 	server.m_comments = xrc_call(*this, "ID_COMMENTS", &wxTextCtrl::GetValue);
 
 	const wxString serverType = xrc_call(*this, "ID_SERVERTYPE", &wxChoice::GetStringSelection);
@@ -1429,6 +1465,38 @@ bool CSiteManagerDialog::GetServer(CSiteManagerItemData_Site& data)
 	return true;
 }
 
+void CSiteManagerDialog::OnKeyFileBrowse(wxCommandEvent& event)
+{
+	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
+	if (!pTree)
+		return;
+
+	wxTreeItemId item = pTree->GetSelection();
+	if (!item.IsOk())
+		return;
+
+	CSiteManagerItemData* data = static_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
+	if (!data)
+		return;
+
+	wxString wildcards(_T("PPK files|*.ppk|PEM files|*.pem|All files|*.*"));
+	wxFileDialog dlg(this, _("Choose a key file"), wxString(), wxString(), wildcards, wxFD_OPEN|wxFD_FILE_MUST_EXIST);
+
+	if (dlg.ShowModal() == wxID_OK) {
+		wxString keyFilePath = dlg.GetPath();
+		// If the selected file was a PEM file, LoadKeyFile() will automatically convert it to PPK
+		// and tell us the new location.
+		CFZPuttyGenInterface fzpg(this);
+
+		wxString keyFileComment, keyFileData;
+		if (fzpg.LoadKeyFile(keyFilePath, false, keyFileComment, keyFileData)) {
+			XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->ChangeValue(keyFilePath);
+		}
+		else {
+			xrc_call(*this, "ID_KEYFILE", &wxWindow::SetFocus);
+		}
+	}
+}
 void CSiteManagerDialog::OnRemoteDirBrowse(wxCommandEvent& event)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
@@ -1444,8 +1512,7 @@ void CSiteManagerDialog::OnRemoteDirBrowse(wxCommandEvent& event)
 		return;
 
 	wxDirDialog dlg(this, _("Choose the default local directory"), XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->GetValue(), wxDD_NEW_DIR_BUTTON);
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->ChangeValue(dlg.GetPath());
 	}
 }
@@ -1505,7 +1572,7 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_NEWFOLDER", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWSITE", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWBOOKMARK", wxWindow)->Enable(false);
-		XRCCTRL(*this, "ID_CONNECT", wxWindow)->Enable(false);
+		XRCCTRL(*this, "ID_CONNECT", wxButton)->Enable(false);
 
 		// Empty all site information
 		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(wxString());
@@ -1516,8 +1583,11 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->ChangeValue(wxString());
 		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->ChangeValue(wxString());
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->ChangeValue(wxString());
+		XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->ChangeValue(wxString());
 		XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->ChangeValue(wxString());
 
+		SetControlVisibility(FTP, ANONYMOUS);
+
 		XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetSelection(0);
 		XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->ChangeValue(wxString());
 		XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->ChangeValue(wxString());
@@ -1550,7 +1620,7 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_NEWFOLDER", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWSITE", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWBOOKMARK", wxWindow)->Enable(!predefined);
-		XRCCTRL(*this, "ID_CONNECT", wxWindow)->Enable(true);
+		XRCCTRL(*this, "ID_CONNECT", wxButton)->Enable(true);
 
 		XRCCTRL(*this, "ID_HOST", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(site_data->m_server.FormatHost(true));
@@ -1570,6 +1640,10 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->Enable(!predefined && site_data->m_server.GetLogonType() != ANONYMOUS);
 		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->Enable(!predefined && (site_data->m_server.GetLogonType() == NORMAL || site_data->m_server.GetLogonType() == ACCOUNT));
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->Enable(!predefined && site_data->m_server.GetLogonType() == ACCOUNT);
+		XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->Enable(!predefined && site_data->m_server.GetLogonType() == KEY);
+		XRCCTRL(*this, "ID_KEYFILE_BROWSE", wxButton)->Enable(!predefined && site_data->m_server.GetLogonType() == KEY);
+
+		SetControlVisibility(site_data->m_server.GetProtocol(), site_data->m_server.GetLogonType());
 
 		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(CServer::GetNameFromLogonType(site_data->m_server.GetLogonType()));
 		XRCCTRL(*this, "ID_LOGONTYPE", wxWindow)->Enable(!predefined);
@@ -1577,6 +1651,7 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->ChangeValue(site_data->m_server.GetUser());
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->ChangeValue(site_data->m_server.GetAccount());
 		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->ChangeValue(site_data->m_server.GetPass());
+		XRCCTRL(*this, "ID_KEYFILE", wxTextCtrl)->ChangeValue(site_data->m_server.GetKeyFile());
 		XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->ChangeValue(site_data->m_comments);
 		XRCCTRL(*this, "ID_COMMENTS", wxWindow)->Enable(!predefined);
 
@@ -1609,19 +1684,16 @@ void CSiteManagerDialog::SetCtrlState()
 		int maxMultiple = site_data->m_server.MaximumMultipleConnections();
 		XRCCTRL(*this, "ID_LIMITMULTIPLE", wxCheckBox)->SetValue(maxMultiple != 0);
 		XRCCTRL(*this, "ID_LIMITMULTIPLE", wxWindow)->Enable(!predefined);
-		if (maxMultiple != 0)
-		{
+		if (maxMultiple != 0) {
 			XRCCTRL(*this, "ID_MAXMULTIPLE", wxSpinCtrl)->Enable(!predefined);
 			XRCCTRL(*this, "ID_MAXMULTIPLE", wxSpinCtrl)->SetValue(maxMultiple);
 		}
-		else
-		{
+		else {
 			XRCCTRL(*this, "ID_MAXMULTIPLE", wxSpinCtrl)->Enable(false);
 			XRCCTRL(*this, "ID_MAXMULTIPLE", wxSpinCtrl)->SetValue(1);
 		}
 
-		switch (site_data->m_server.GetEncodingType())
-		{
+		switch (site_data->m_server.GetEncodingType()) {
 		default:
 		case ENCODING_AUTO:
 			XRCCTRL(*this, "ID_CHARSET_AUTO", wxRadioButton)->SetValue(true);
@@ -1642,8 +1714,7 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_CONNECT", wxButton)->SetDefault();
 #endif
 	}
-	else
-	{
+	else {
 		m_pNotebook_Site->Hide();
 		m_pNotebook_Bookmark->Show();
 		m_pNotebook_Site->GetContainingSizer()->Layout();
@@ -1655,7 +1726,7 @@ void CSiteManagerDialog::SetCtrlState()
 		XRCCTRL(*this, "ID_NEWFOLDER", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWSITE", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_NEWBOOKMARK", wxWindow)->Enable(!predefined);
-		XRCCTRL(*this, "ID_CONNECT", wxWindow)->Enable(true);
+		XRCCTRL(*this, "ID_CONNECT", wxButton)->Enable(true);
 
 		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->ChangeValue(data->m_localDir);
 		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxWindow)->Enable(!predefined);
@@ -1687,12 +1758,45 @@ void CSiteManagerDialog::OnCharsetChange(wxCommandEvent& event)
 
 void CSiteManagerDialog::OnProtocolSelChanged(wxCommandEvent& event)
 {
-	wxChoice* pProtocol = XRCCTRL(*this, "ID_PROTOCOL", wxChoice);
-	wxChoice* pEncryption = XRCCTRL(*this, "ID_ENCRYPTION", wxChoice);
-	wxStaticText* pEncryptionDesc = XRCCTRL(*this, "ID_ENCRYPTION_DESC", wxStaticText);
+	SetControlVisibility(GetProtocol(), GetLogonType());
+}
 
-	pEncryption->Show(pProtocol->GetSelection() != 1);
-	pEncryptionDesc->Show(pProtocol->GetSelection() != 1);
+void CSiteManagerDialog::SetControlVisibility(ServerProtocol protocol, LogonType type)
+{
+	xrc_call(*this, "ID_ENCRYPTION_DESC", &wxStaticText::Show, protocol != SFTP);
+	xrc_call(*this, "ID_ENCRYPTION", &wxChoice::Show, protocol != SFTP);
+
+	auto choice = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice);
+
+	// Disallow invalid combinations
+	if (protocol == SFTP && type == ACCOUNT) {
+		type = NORMAL;
+		choice->Select(choice->FindString(CServer::GetNameFromLogonType(type)));
+	}
+	else if (protocol != SFTP && type == KEY) {
+		type = NORMAL;
+		choice->Select(choice->FindString(CServer::GetNameFromLogonType(type)));
+	}
+
+	int toDelete = choice->FindString(CServer::GetNameFromLogonType(protocol == SFTP ? ACCOUNT : KEY));
+	int toAdd    = choice->FindString(CServer::GetNameFromLogonType(protocol != SFTP ? ACCOUNT : KEY));
+	if (toDelete != -1) {
+		choice->Delete(toDelete);
+	}
+	if (toAdd == -1) {
+		choice->Append(CServer::GetNameFromLogonType(protocol != SFTP ? ACCOUNT : KEY));
+	}
+
+	xrc_call(*this, "ID_ACCOUNT_DESC", &wxStaticText::Show, protocol != SFTP && type == ACCOUNT);
+	xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::Show, protocol != SFTP && type == ACCOUNT);
+	xrc_call(*this, "ID_PASS_DESC", &wxStaticText::Show, protocol != SFTP || type != KEY);
+	xrc_call(*this, "ID_PASS", &wxTextCtrl::Show, protocol != SFTP || type != KEY);
+	xrc_call(*this, "ID_KEYFILE_DESC", &wxStaticText::Show, protocol == SFTP && type == KEY);
+	xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::Show, protocol == SFTP && type == KEY);
+	xrc_call(*this, "ID_KEYFILE_BROWSE", &wxButton::Show, protocol == SFTP && type == KEY);
+
+	xrc_call(*this, "ID_KEYFILE_DESC", &wxStaticText::GetContainingSizer)->CalcMin();
+	xrc_call(*this, "ID_KEYFILE_DESC", &wxStaticText::GetContainingSizer)->Layout();
 }
 
 void CSiteManagerDialog::OnCopySite(wxCommandEvent& event)
@@ -1778,12 +1882,12 @@ bool CSiteManagerDialog::LoadDefaultSites()
 
 	CXmlFile file(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument)
+	auto document = file.Load();
+	if (!document)
 		return false;
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
+	auto element = document.child("Servers");
+	if (!element)
 		return false;
 
 	wxTreeCtrlEx *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrlEx);
@@ -1809,7 +1913,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 		lastSelection.clear();
 	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection, true);
 
-	CSiteManager::Load(pElement, handler);
+	CSiteManager::Load(element, handler);
 
 	return true;
 }
@@ -2110,10 +2214,10 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 
 	CXmlFile xml(dlg.GetPath());
 
-	TiXmlElement* exportRoot = xml.CreateEmpty();
+	auto exportRoot = xml.CreateEmpty();
 
-	TiXmlElement* pServers = exportRoot->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
-	SaveChild(pServers, m_contextMenuItem);
+	auto servers = exportRoot.append_child("Servers");
+	SaveChild(servers, m_contextMenuItem);
 
 	if (!xml.Save(false)) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), xml.GetFileName(), xml.GetError());
@@ -2260,3 +2364,8 @@ ServerProtocol CSiteManagerDialog::GetProtocol() const
 		return INSECURE_FTP;
 	}
 }
+
+LogonType CSiteManagerDialog::GetLogonType() const
+{
+	return CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
+}
