@@ -150,21 +150,20 @@ bool CUpdater::LongTimeSinceLastCheck() const
 	if (lastCheckStr.empty())
 		return true;
 
-	wxDateTime lastCheck;
-	lastCheck.ParseDateTime(lastCheckStr);
+	CDateTime lastCheck(lastCheckStr, CDateTime::utc);
 	if (!lastCheck.IsValid())
 		return true;
 
-	wxTimeSpan const span = wxDateTime::Now() - lastCheck;
+	auto const span = CDateTime::Now() - lastCheck;
 
-	if (span.GetSeconds() < 0)
+	if (span.get_seconds() < 0)
 		// Last check in future
 		return true;
 
 	int days = 1;
 	if (!CBuildInfo::IsUnstable())
 		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
-	return span.GetDays() >= days;
+	return span.get_days() >= days;
 }
 
 wxString CUpdater::GetUrl()
@@ -206,11 +205,11 @@ bool CUpdater::Run()
 		return false;
 	}
 
-	wxDateTime const t = wxDateTime::Now();
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.Format(_T("%Y-%m-%d %H:%M:%S")));
+	auto  const t = CDateTime::Now();
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.Format(_T("%Y-%m-%d %H:%M:%S"), CDateTime::utc));
 
 	local_file_.clear();
-	log_ = wxString::Format(_("Started update check on %s\n"), t.Format());
+	log_ = wxString::Format(_("Started update check on %s\n"), t.Format(_T("%Y-%m-%d %H:%M:%S"), CDateTime::local));
 
 	wxString build = CBuildInfo::GetBuildType();
 	if( build.empty() ) {
@@ -380,8 +379,8 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 			}
 			else {
 				s = UpdaterState::newversion_downloading;
-				wxLongLong size = CLocalFileSystem::GetSize(temp);
-				if( size >= 0 && static_cast<unsigned long long>(size.GetValue()) >= version_information_.available_.size_ ) {
+				auto size = CLocalFileSystem::GetSize(temp);
+				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
 				else if( !can_download || Download( version_information_.available_.url_, GetTempFile() ) != FZ_REPLY_WOULDBLOCK ) {
@@ -466,7 +465,7 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 	return s;
 }
 
-wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
+wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 {
 	wxString const fn = GetFilename( b.url_ );
 	wxString const dl = GetDownloadDir().GetPath();
@@ -578,15 +577,15 @@ void CUpdater::ParseData()
 		wxString versionOrDate = tokens.GetNextToken();
 
 		if (type == _T("nightly")) {
-			wxDateTime nightlyDate;
-			if( !nightlyDate.ParseFormat(versionOrDate, _T("%Y-%m-%d")) ) {
-				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+			CDateTime nightlyDate(versionOrDate, CDateTime::utc);
+			if (!nightlyDate.IsValid()) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += _T("Could not parse nightly date\n");
 				}
 				continue;
 			}
 
-			wxDateTime buildDate = CBuildInfo::GetBuildDate();
+			CDateTime buildDate = CBuildInfo::GetBuildDate();
 			if (!buildDate.IsValid() || !nightlyDate.IsValid() || nightlyDate <= buildDate) {
 				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
 					log_ += _T("Nightly isn't newer\n");
@@ -659,14 +658,14 @@ void CUpdater::OnTimer(wxTimerEvent&)
 	AutoRunIfNeeded();
 }
 
-bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString const& checksum )
+bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum)
 {
-	if( file.empty() || checksum.empty() ) {
+	if (file.empty() || checksum.empty()) {
 		return false;
 	}
 
-	wxLongLong filesize = CLocalFileSystem::GetSize(file);
-	if( filesize < 0 || static_cast<unsigned long long>(filesize.GetValue()) != size ) {
+	auto filesize = CLocalFileSystem::GetSize(file);
+	if (filesize < 0 || filesize != size) {
 		return false;
 	}
 
@@ -693,7 +692,7 @@ bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString
 	SHA512_Final(&state, raw_digest);
 
 	wxString digest;
-	for( unsigned int i = 0; i < sizeof(raw_digest); i++ ) {
+	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
 		unsigned char l = raw_digest[i] >> 4;
 		unsigned char r = raw_digest[i] & 0x0F;
 
@@ -736,7 +735,7 @@ wxString CUpdater::GetFilename( wxString const& url) const
 {
 	wxString ret;
 	int pos = url.Find('/', true);
-	if( pos != -1 ) {
+	if (pos != -1) {
 		ret = url.Mid(pos + 1);
 	}
 	size_t p = ret.find_first_of(_T("?#"));
@@ -803,11 +802,11 @@ void CUpdater::RemoveHandler( CUpdateHandler& handler )
 	}
 }
 
-wxULongLong CUpdater::BytesDownloaded() const
+int64_t CUpdater::BytesDownloaded() const
 {
-	wxLongLong ret;
-	if( state_ == UpdaterState::newversion_ready ) {
-		if( !local_file_.empty() ) {
+	int64_t ret{-1};
+	if (state_ == UpdaterState::newversion_ready) {
+		if (!local_file_.empty()) {
 			ret = CLocalFileSystem::GetSize(local_file_);
 		}
 	}
@@ -817,10 +816,7 @@ wxULongLong CUpdater::BytesDownloaded() const
 			ret = CLocalFileSystem::GetSize(temp);
 		}
 	}
-	if( ret < 0 ) {
-		ret = 0;
-	}
-	return static_cast<unsigned long long>(ret.GetValue());
+	return ret;
 }
 
 bool CUpdater::UpdatableBuild() const
